//=============================================================================
// Factory class for spawning an MG42 machine gun.  Doubles as the weapon's skeletal
// mesh when viewed in third-person mode.
//
// Author: Neil Johnson, Gearbox Software.
//=============================================================================
class MG42Factory extends IMG42Factory
	placeable;


//---------------------------------------------------------
// Properties
//---------------------------------------------------------
var (MG42Factory)	Name	GunnerMountTag,
							FeederMountTag,
							SpotterMountTag;
var (MG42Factory)	Name	MountEvent,
							DismountEvent;
var (MG42Factory)	Name	GunnerToDraft;
var (MG42Factory)	bool	IgnoreSuppression;			// Gunner's behavior doesn't change with suppression.
var (MG42Factory)	bool	DontRecruit;
var (MG42Factory)	Range	PauseTime_Low;
var (MG42Factory)	Range	PauseTime_High;
var	(MG42Factory)	Range	BurstLength_Low;
var	(MG42Factory)	Range	BurstLength_High;
var (MG42Factory)	float	EnemyTooCloseDist;

//---------------------------------------------------------
// Variables
//---------------------------------------------------------
var TurretWeapon			Gun;			// The weapon itself;
var gbxContextUseTrigger	TurretTrigger;	// Trigger to allow the player to mount the gun.

var Name	CurNinePointAnim1, CurNinePointAnim2;
var Name	ForwardDownAnim, ForwardLevelAnim, ForwardUpAnim,
			LeftDownAnim, LeftLevelAnim, LeftUpAnim,
			RightDownAnim, RightLevelAnim, RightUpAnim;

const NINEPOINT_CHANNEL0 = 0;
const NINEPOINT_CHANNEL1 = 1;
const NINEPOINT_CHANNEL2 = 2;

var const Vector	GunOffset;		// Offset at which the gun is positioned
var Vector PivotOrigin, PivotOffset;

//---------------------------------------------------------
//	SpawnGun
//---------------------------------------------------------
function SpawnGun()
{
	local vector X, Y, Z;

	GetAxes(Rotation,X,Y,Z);
	Gun = Spawn(class'WeapDEMG42_', Self,, Location, Rotation);
	PivotOrigin = Location + PivotOffset.X*X + PivotOffset.Y*Y + PivotOffset.Z*Z;
	Gun.SetLocation( PivotOrigin + GunOffset.X*X + GunOffset.Y*Y + GunOffset.Z*Z );
	WeapDEMG42_( Gun ).SetFactory( Self );
}

//---------------------------------------------------------
//	Rebalance
//---------------------------------------------------------
function Vector Rebalance( optional Rotator Rot)
{
	local vector X, Y, Z;

	if ( Rot == Rot(0,0,0) )
		Rot = Rotation;

	GetAxes(Rot,X,Y,Z);

	return PivotOrigin + GunOffset.X*X + GunOffset.Y*Y + GunOffset.Z*Z;
}

//---------------------------------------------------------
//	PostNetBeginPlay
//---------------------------------------------------------
function PostBeginPlay()
{	
	local WeapDEMG42_ TheMG;

	Super.PostBeginPlay();

	SpawnGun();

	TheMG = WeapDEMG42_( Gun );

	// Impart properties to the MG42.
	TheMG.Zone1OuterRadius = Zone1OuterRadius;
	TheMG.Zone2OuterRadius = Zone2OuterRadius;
	TheMG.Zone3OuterRadius = Zone3OuterRadius;

	TheMG.GunnerToDraft = GunnerToDraft;
	TheMG.GunnerMountTag = GunnerMountTag;
	TheMG.MountEvent = MountEvent;
	TheMG.DismountEvent = DismountEvent;

	TheMG.bIgnoreSuppression = IgnoreSuppression;
	TheMG.NoShotProtectionDist = NoShotProtectionDist;

	TheMG.bDontRecruit = DontRecruit;

	TheMG.AttachPathNodes();

	if ( Self.Tag != '' && Self.Tag != 'MG42Factory' )
	{
		Gun.Tag = Self.Tag;
	}
	else
	{
		Gun.Tag = Gun.Name;		// At the least, give the weapon a unique tag name.
	}

	TheMG.PauseTime_Low = PauseTime_Low;
	TheMG.PauseTime_High = PauseTime_High;

	InstantFire( TheMG.FireMode[0] ).BurstLength_Low = BurstLength_Low;
	InstantFire( TheMG.FireMode[0] ).BurstLength_High = BurstLength_High;

	TurretTrigger = spawn(class'gbxContextUseTrigger', Gun);
	TurretTrigger.SetCollisionSize( Self.CollisionRadius, Self.CollisionHeight );
	TurretTrigger.m_eUseType=UseMG42;
	TurretTrigger.Event = Gun.Tag;
	TurretTrigger.SetBase(Self);
	TurretTrigger.SetCollision(true, false, false);

	TheMG.MyTrigger = TurretTrigger;
	TheMG.EnemyTooCloseDist = EnemyTooCloseDist;

	SetCollisionSize( 5.0, Self.CollisionHeight );
}

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	
	Mesh = Mesh( DynamicLoadObject( "k_prop.de_mg42_emplaced", class'Mesh' ) );
}

//---------------------------------------------------------
//	ShowFactory
//---------------------------------------------------------
function ShowFactory()
{
	bHidden = false;
	PlayAnim( ForwardLevelAnim,,, NINEPOINT_CHANNEL0);
}

//---------------------------------------------------------
//	HideFactory
//---------------------------------------------------------
function HideFactory()
{
	bHidden = true;
}

//---------------------------------------------------------
//	DoNinePointBlending
//---------------------------------------------------------
function DoNinePointBlending( NinePointDir Dir1, float BlendAlpha1, NinePointDir Dir2, float BlendAlpha2 )
{
	local Name Dir1Anim, Dir2Anim;

	if (BlendAlpha2 > 0.01) {
		AnimBlendParams(NINEPOINT_CHANNEL2, BlendAlpha2, 0.0f, 0);
	}
	if (BlendAlpha1 > 0.01) {
		AnimBlendParams(NINEPOINT_CHANNEL1, BlendAlpha1, 0.0f, 0);
	}
	
	// Dir1 denotes the first animation to blend.  The first animation
	// should be an "edge animation" (aligned with one of the cardinal
	// directions of up, down, left, or right).
	//
	// Dir2 denotes the second animation to blend.
	// It represents a "diagonal animation" (aligned with one
	// of the diagonal directions: e.g. up-left, down-right, etc.
	//
	switch(Dir1) {
		case NPD_UpLeft:
			Dir1Anim = LeftUpAnim;
			break;
		case NPD_UpCenter:
			Dir1Anim = ForwardUpAnim;
			break;
		case NPD_UpRight:
			Dir1Anim = RightUpAnim;
			break;
		//--------------
		case NPD_MidLeft:
			Dir1Anim = LeftLevelAnim;
			break;
		case NPD_MidCenter:
			Dir1Anim = ForwardLevelAnim;
			break;
		case NPD_MidRight:
			Dir1Anim = RightLevelAnim;
			break;
		//--------------
		case NPD_LowLeft:
			Dir1Anim = LeftDownAnim;
			break;
		case NPD_LowCenter:
			Dir1Anim = ForwardDownAnim;
			break;
		case NPD_LowRight:
			Dir1Anim = RightDownAnim;
			break;
	}

	switch(Dir2) {
		case NPD_UpLeft:
			Dir2Anim = LeftUpAnim;
			break;
		case NPD_UpCenter:
			Dir2Anim = ForwardUpAnim;
			break;
		case NPD_UpRight:
			Dir2Anim = RightUpAnim;
			break;
		//--------------
		case NPD_MidLeft:
			Dir2Anim = LeftLevelAnim;
			break;
		case NPD_MidCenter:
			Dir2Anim = ForwardLevelAnim;
			break;
		case NPD_MidRight:
			Dir2Anim = RightLevelAnim;
			break;
		//--------------
		case NPD_LowLeft:
			Dir2Anim = LeftDownAnim;
			break;
		case NPD_LowCenter:
			Dir2Anim = ForwardDownAnim;
			break;
		case NPD_LowRight:
			Dir2Anim = RightDownAnim;
			break;
	}

	// Play the new Dir1 animation if it has changed.
	if (Dir1Anim != CurNinePointAnim1) {
		CurNinePointAnim1 = Dir1Anim;
		LoopAnim(CurNinePointAnim1, 1.0, 0.1, NINEPOINT_CHANNEL1);
	}

	// Play the new Dir2 animation if it has changed.
	if (Dir2Anim != CurNinePointAnim2) {
		CurNinePointAnim2 = Dir2Anim;
		LoopAnim(CurNinePointAnim2, 1.0, 0.1, NINEPOINT_CHANNEL2);
	}
}

function StopBlending()
{
	AnimBlendParams(NINEPOINT_CHANNEL2, 0.0f, 0.0f, 0);
	AnimBlendParams(NINEPOINT_CHANNEL1, 0.0f, 0.0f, 0);
}

//---------------------------------------------------------
//	Landed
//---------------------------------------------------------
event Landed( vector HitLocation, vector HitNormal )
{
	local vector X, Y, Z;

	GetAxes(Rotation,X,Y,Z);
	PivotOrigin = Location + PivotOffset.X*X + PivotOffset.Y*Y + PivotOffset.Z*Z;
	Gun.SetLocation( PivotOrigin + GunOffset.X*X + GunOffset.Y*Y + GunOffset.Z*Z );
	Gun.Landed( HitLocation, HitNormal );
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	DrawType=DT_MESH
//	Mesh=k_prop.de_mg42_emplaced
	EditorMeshName="k_prop.de_mg42_emplaced"

	bHidden=false

	CollisionRadius=140.0
	CollisionHeight=40

	Zone1OuterRadius=2000.0
	Zone2OuterRadius=2500.0
	Zone3OuterRadius=3000.0
	NoShotProtectionDist=1000.0

	PivotOffset=(X=77,Y=0,Z=30)
	GunOffset=(X=28,Y=0,Z=17)

	PauseTime_Low=(Min=0.5,Max=0.8)
	PauseTime_High=(Min=3.0,Max=5.0)

	BurstLength_Low=(Min=0.5,Max=1.0)
//	BurstLength_Med=(Min=0.5,Max=1.0)
	BurstLength_High=(Min=0.0,Max=0.3)

	IgnoreSuppression=false

	MountEvent=MG42_mount
	DismountEvent=MG42_dismount

	// Nine-point blending animations.
	ForwardDownAnim=f_down
	ForwardLevelAnim=f_level
	ForwardUpAnim=f_up
	LeftDownAnim=l_down
	LeftLevelAnim=l_level
	LeftUpAnim=l_up
	RightDownAnim=r_down
	RightLevelAnim=r_level
	RightUpAnim=r_up

	EnemyTooCloseDist=1000.0

	//Physics=PHYS_Falling
	Physics=PHYS_None
	bStatic=false
	bCollideActors=false
	bBlockActors=false
	bBlockPlayers=false
	bCollideWorld=true
	bCollideSkeletalMesh=true
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true

	bOpaqueForSituationalAwarenessView=true

	bAlphaFogable=True
    bAlphaFogBlend=true
}
